from geda.data_providers.duts import DUTSDataProvider
from geda.datasets.base import BaseSegmentationDataset
from typing import Literal


class DUTSSegmentationDataset(BaseSegmentationDataset):
    def __init__(
        self,
        root: str,
        split: Literal["train", "test"],
        labels_format: Literal["yolo"] = "yolo",
    ):
        data_provider = DUTSDataProvider(root, binarize=True, labels_format=labels_format)
        super().__init__(root, split, data_provider)


if __name__ == "__main__":
    from geda.utils.config import ROOT

    root = str(ROOT / "data" / "DUTS")
    ds = DUTSSegmentationDataset(root, "train")
    ds = DUTSSegmentationDataset(root, "test")



from tqdm import tqdm
import urllib.request
import tarfile
from geda.utils.pylogger import get_pylogger
import os
import shutil
from pathlib import Path
import zipfile
import yaml
import gzip

log = get_pylogger(__name__)


class TqdmUpTo(tqdm):
    """Provides `update_to(n)` which uses `tqdm.update(delta_n)`."""

    def update_to(self, b=1, bsize=1, tsize=None):
        """
        b  : int, optional
            Number of blocks transferred so far [default: 1].
        bsize  : int, optional
            Size of each block (in tqdm units) [default: 1].
        tsize  : int, optional
            Total size (in tqdm units). If [default: None] remains unchanged.
        """
        if tsize is not None:
            self.total = tsize
        return self.update(b * bsize - self.n)  # also sets self.n = b * bsize


def download_file(url, filepath):
    log.info(f"Downloading {url} to {filepath}")
    with TqdmUpTo(
        unit="B",
        unit_scale=True,
        unit_divisor=1024,
        miniters=1,
        desc=url.split("/")[-1],
    ) as t:  # all optional kwargs
        urllib.request.urlretrieve(
            url, filename=filepath, reporthook=t.update_to, data=None
        )
        t.total = t.n
    log.info("Download finished")


def unzip_tar(file_path, dst_path, mode: str = "r"):
    with tarfile.open(file_path, mode) as tar:
        tar.extractall(dst_path)
    log.info("Unzipping finished")


def unzip_zip(file_path, dst_path):
    with zipfile.ZipFile(file_path, "r") as zip_ref:
        zip_ref.extractall(dst_path)


def unzip_gz(filepath):
    with gzip.open(filepath, "rb") as f_in:
        with open(filepath.replace(".gz", ""), "wb") as f_out:
            shutil.copyfileobj(f_in, f_out)


def unzip(file_path, dst_path, remove=False):
    log.info(f"Unzipping {file_path} to {dst_path}.")
    ext = file_path.split(".")[-1]
    if ext == "tar":
        unzip_tar(file_path, dst_path)
    elif ext == "zip":
        unzip_zip(file_path, dst_path)
    elif ext == "gz":
        unzip_gz(file_path)
    if remove:
        os.remove(file_path)
        log.info(f"Removed {file_path}")


def save_txt_to_file(txt, filename):
    with open(filename, "w") as file:
        file.write(txt)


def read_txt_file(filename) -> list[str]:
    with open(filename, "r") as file:
        lines = file.readlines()
        lines = [
            line.strip() for line in lines
        ]  # Optional: Remove leading/trailing whitespace
    return lines


def add_prefix_to_files(directory, prefix, ext=".png"):
    log.info(f"Adding {prefix} prefix to all {ext} files in {directory} directory")

    for filename in os.listdir(directory):
        if os.path.isfile(os.path.join(directory, filename)) and filename.endswith(ext):
            new_filename = prefix + filename
            os.rename(
                os.path.join(directory, filename), os.path.join(directory, new_filename)
            )
    log.info("Prefix addition finished")


def move(source, destination):
    shutil.move(source, destination)
    log.info(f"Moved {source} to {destination}")


def copy_directory(source_dir, destination_dir):
    shutil.copytree(source_dir, destination_dir)
    log.info(f"Copied {source_dir} to {destination_dir}")


def remove_directory(dir_path):
    shutil.rmtree(dir_path)
    log.info(f"Removed {dir_path} directory")


def path_exists(path: str):
    return os.path.exists(path)


def copy_all_files(source_dir, destination_dir, ext=".png"):
    filenames = os.listdir(source_dir)
    for filename in tqdm(filenames, desc="Copying files"):
        if filename.lower().endswith(ext):
            source = source_dir / filename
            destination = destination_dir / filename
            shutil.copy2(source, destination)
    log.info(
        f"Copied all {ext} files ({len(filenames)}) from {source_dir} to {destination_dir}"
    )


def copy_files(source_filepaths, dest_filepaths):
    for source_filepath, dest_filepath in tqdm(
        zip(source_filepaths, dest_filepaths), desc="Copying files"
    ):
        try:
            shutil.copy2(source_filepath, dest_filepath)
        except FileNotFoundError as e:
            log.warn(f"{source_filepath} not found")
    log.info(f"Copied files ({len(source_filepaths)})")


def create_dir(path: Path, return_str: bool = True):
    path.mkdir(parents=True, exist_ok=True)
    if return_str:
        return str(path)
    return path


def load_yaml(path: Path | str) -> dict:
    with open(path, "r") as file:
        yaml_dct = yaml.safe_load(file)
    return yaml_dct


def save_yaml(dct: dict, path: Path | str):
    with open(path, "w") as file:
        yaml.dump(dct, file)



from geda.data_providers.base import ClassificationDataProvider
from geda.utils.files import move, copy_files, create_dir, save_txt_to_file
from geda.utils.pylogger import get_pylogger
from tqdm.auto import tqdm
import numpy as np
from pathlib import Path
from PIL import Image

log = get_pylogger(__name__)


_URLS = {
    "train_images": "http://yann.lecun.com/exdb/mnist/train-images-idx3-ubyte.gz",
    "train_labels": "http://yann.lecun.com/exdb/mnist/train-labels-idx1-ubyte.gz",
    "test_images": "http://yann.lecun.com/exdb/mnist/t10k-images-idx3-ubyte.gz",
    "test_labels": "http://yann.lecun.com/exdb/mnist/t10k-labels-idx1-ubyte.gz",
}

SPLIT_NUM_SAMPLES = {"train": 60_000, "test": 10_000}


SPLITS = ["train", "test"]


class MNISTDataProvider(ClassificationDataProvider):
    URL = "http://yann.lecun.com/exdb/mnist/"

    def __init__(self, root: str):
        self.task = "Classification"
        self.task_root = f"{root}/{self.task}"
        super().__init__(urls=_URLS, root=root)

    def move_to_raw_root(self):
        filepaths = list(_URLS.keys())
        for filepath in filepaths:
            src_path = f"{self.root}/{filepath}"
            dst_path = f"{self.raw_root}/{filepath}"
            move(src_path, dst_path)

    def _get_split_ids(self):
        split_ids = {}
        for split in SPLITS:
            ids = [str(i) for i in range(SPLIT_NUM_SAMPLES[split])]
            split_ids[split] = ids
        return split_ids

    def arrange_files(self):
        def load_ubyte_to_array(filepath, labels=False):
            with open(filepath, "rb") as file:
                # file = open(filepath, "r")
                if labels:
                    file.read(8)
                else:
                    file.read(16)
                buf = file.read()
                data = np.frombuffer(buf, dtype=np.uint8)
                if not labels:
                    data = data.reshape(-1, 28, 28, 1)
                return data

        for split, ids in self.split_ids.items():
            names = ["images", "labels"]
            dst_paths = {name: create_dir(Path(self.task_root) / name / split) for name in names}

            images_ubyte_filepath = f"{self.raw_root}/{split}_images"
            labels_ubyte_filepath = f"{self.raw_root}/{split}_labels"

            images = load_ubyte_to_array(images_ubyte_filepath, labels=False)
            labels = load_ubyte_to_array(labels_ubyte_filepath, labels=True)
            for _id, image, label in tqdm(
                zip(ids, images, labels),
                desc=f"Saving images and labels for {split} split",
                total=len(ids),
            ):
                h, w, c = image.shape
                if c == 1:
                    image = image.squeeze(2)
                dst_label_filepath = f"{dst_paths['labels']}/{_id}.txt"
                dst_image_filepath = f"{dst_paths['images']}/{_id}.jpg"

                Image.fromarray(image).save(dst_image_filepath)
                save_txt_to_file(str(label), dst_label_filepath)


if __name__ == "__main__":
    from geda.utils.config import ROOT

    root = str(ROOT / "data" / "MNIST")
    dp = MNISTDataProvider(root)
    dp.get_data()



from geda.data_providers.mnist import MNISTDataProvider
from geda.datasets.base import BaseClassificationDataset
from typing import Literal


class MNISTClassificationDataset(BaseClassificationDataset):
    def __init__(self, root: str, split: Literal["train", "test"], download: bool = True):
        data_provider = MNISTDataProvider(root)
        super().__init__(root, split, data_provider, download)


if __name__ == "__main__":
    from geda.utils.config import ROOT

    root = str(ROOT / "data" / "DUTS")
    ds = MNISTClassificationDataset(root, "train")
    ds = MNISTClassificationDataset(root, "test")



from geda.get_data import DATA_PROVIDERS, get_data
from geda.utils.config import DATA_PATH

ROOTS = {
    "DUTS": DATA_PATH / "DUTS",
    "NYUDv2": DATA_PATH / "NYUDv2",
    "VOC": DATA_PATH / "voc_2012",
}


def _download(name: str):
    root_name, *_ = name.split("_")
    root = str(ROOTS[root_name])
    dp = get_data(name, root=root)
    print(dp.class_frequencies)
    print(dp.class_counts)


def test_download():
    for name in DATA_PROVIDERS:
        _download(name)


if __name__ == "__main__":
    # test_download()
    _download("DUTS")
    # _download("VOC_SemanticSegmentationAug")



from geda.data_providers.base import SegmentationDataProvider
from geda.utils.files import (
    move,
    copy_files,
    create_dir,
)
from geda.utils.images import save_gray_array_as_color_png
from geda.parsers.yolo import parse_segmentation_masks_to_yolo
from pathlib import Path
import glob
from geda.utils.pylogger import get_pylogger
from PIL import Image
import numpy as np
from tqdm.auto import tqdm
import os
from typing import Literal


log = get_pylogger(__name__)


_URLS = {
    "train": "http://saliencydetection.net/duts/download/DUTS-TR.zip",
    "test": "http://saliencydetection.net/duts/download/DUTS-TE.zip",
}

split2dirname = {"train": "DUTS-TR", "test": "DUTS-TE"}

_ID2CLASS = {0: "background", 1: "salient"}
_PALETTE = [(0, 0, 0), (255, 255, 255)]  # black for background, white for salient

SPLITS = ["train", "test"]


class DUTSDataProvider(SegmentationDataProvider):
    URL = "http://saliencydetection.net/duts/"

    def __init__(
        self,
        root: str,
        binarize: bool = True,
        labels_format: Literal["yolo"] | None = None,
    ):
        self.task = "Segmentation"
        self.task_root = f"{root}/{self.task}"
        self.binarize = binarize
        super().__init__(urls=_URLS, root=root, labels_format=labels_format)

    def move_to_raw_root(self):
        for split in SPLITS:
            src_dir = f"{self.root}/{split2dirname[split]}/"
            move(src_dir, self.raw_root)

    def _get_split_ids(self):
        split_ids = {}
        for split in SPLITS:
            split_name = split2dirname[split]
            filepaths = sorted(glob.glob(f"{self.raw_root}/{split_name}/{split_name}-Mask/*png"))
            ids = [os.path.basename(path).split(".")[0] for path in filepaths]
            split_ids[split] = ids
        return split_ids

    def arrange_files(self):
        for split, ids in self.split_ids.items():
            names = ["masks", "images", "labels"]
            dst_paths = {name: create_dir(Path(self.task_root) / name / split) for name in names}
            self._set_id2class(id2class=_ID2CLASS)
            split_name = split2dirname[split]
            src_paths = {
                "masks": f"{self.raw_root}/{split_name}/{split_name}-Mask",
                "images": f"{self.raw_root}/{split_name}/{split_name}-Image",
            }

            src_masks_filepaths = [f"{src_paths['masks']}/{_id}.png" for _id in ids]

            src_images_filepaths = [f"{src_paths['images']}/{_id}.jpg" for _id in ids]

            dst_masks_filepaths = [f"{dst_paths['masks']}/{_id}.png" for _id in ids]
            dst_images_filepaths = [f"{dst_paths['images']}/{_id}.jpg" for _id in ids]

            log.info(f"Moving {split} masks from {src_paths['masks']} to {dst_paths['masks']}")
            log.info(f"Moving {split} images from {src_paths['images']} to {dst_paths['images']}")
            copy_files(src_masks_filepaths, dst_masks_filepaths)
            copy_files(src_images_filepaths, dst_images_filepaths)

            if self.binarize:
                log.info(f"Applying binarization (threshold = 128) for {split} masks")
                for mask_filepath in tqdm(dst_masks_filepaths, desc="Binarization"):
                    mask = np.array(Image.open(mask_filepath).convert("L"))
                    binary_mask = ((mask > 128) * 1).astype(np.uint8)
                    save_gray_array_as_color_png(
                        binary_mask, palette=_PALETTE, filename=mask_filepath
                    )

    def create_labels(self):
        masks_filepaths = sorted(glob.glob(f"{self.task_root}/masks/*/*"))
        if self.labels_format == "yolo":
            parse_segmentation_masks_to_yolo(masks_filepaths, background=0, contour=None)
        else:
            log.warn(f"Only YOLO label format is implemented ({self.labels_format} passed)")


if __name__ == "__main__":
    from geda.utils.config import ROOT

    root = str(ROOT / "data" / "DUTS")
    dp = DUTSDataProvider(root)



from geda.data_providers.base import DataProvider

from geda.utils.files import copy_files, save_yaml, create_dir, move
from pathlib import Path
from geda.utils.pylogger import get_pylogger
import scipy
from dataclasses import dataclass
from PIL import Image
from typing import Optional
import numpy as np
import cv2
import matplotlib.pyplot as plt

log = get_pylogger(__name__)

_URLS = {
    "train_data_2017": "http://images.cocodataset.org/zips/train2017.zip",
    "val_data_2017": "http://images.cocodataset.org/zips/val2017.zip",
    "test_data_2017": "http://images.cocodataset.org/zips/test2017.zip",
    "train_val_annots_2017": "http://images.cocodataset.org/annotations/annotations_trainval2017.zip",
}

LABELS = [
    "nose",
    "left_eye",
    "right_eye",
    "left_ear",
    "right_ear",
    "left_shoulder",
    "right_shoulder",
    "left_elbow",
    "right_elbow",
    "left_wrist",
    "right_wrist",
    "left_hip",
    "right_hip",
    "left_knee",
    "right_knee",
    "left_ankle",
    "right_ankle",
]

LIMBS = []


@dataclass
class PointAnnotation:
    x: int
    y: int
    id: int
    is_visible: bool

    @property
    def xy(self) -> tuple[int, int]:
        return self.x, self.y

    @classmethod
    def from_dict(cls, annot: dict) -> "PointAnnotation":
        return cls(**annot)


@dataclass
class PoseAnnotation:
    scale: float
    objpos_xy: tuple[int, int]
    head_xyxy: tuple[int, int, int, int]
    keypoints: list[PointAnnotation]

    @classmethod
    def from_dict(cls, annot: dict):
        scale = annot["scale"]
        objpos_xy = annot["objpos_xy"]
        head_xyxy = annot["head_xyxy"]
        keypoints = {
            point["id"]: PointAnnotation.from_dict(point)
            for point in annot["keypoints"]
        }
        return cls(scale, objpos_xy, head_xyxy, keypoints)


@dataclass
class Annotation:
    filename: str
    is_valid: bool
    poses: Optional[list[PoseAnnotation]] | None = None

    @classmethod
    def from_dict(cls, annot: dict):
        filename = annot["filename"]
        is_valid = annot["is_valid"]
        if not is_valid:
            return cls(filename, is_valid)
        poses = [PoseAnnotation.from_dict(obj_rect) for obj_rect in annot["objects"]]
        return cls(filename, is_valid, poses)

    def plot(self):
        if not self.is_valid or self.poses is None:
            return
        image = np.array(Image.open(f"images/{self.filename}"))
        for pose in self.poses:
            keypoints = pose.keypoints
            for _, kp in keypoints.items():
                color = (0, 128, 255) if kp.is_visible else (128, 128, 128)
                cv2.circle(image, kp.xy, 3, color, -1)
            for id_1, id_2 in LIMBS:
                if id_1 not in keypoints or id_2 not in keypoints:
                    continue
                kp_1 = keypoints[id_1]
                kp_2 = keypoints[id_2]
                cv2.line(image, kp_1.xy, kp_2.xy, (50, 255, 50), 4)
        plt.imshow(image)


def parse_coco_annotation(annot) -> dict:
    """
    returns annot dict in form
    {

    }
    """

    annot_dict = {}
    return annot_dict


class COCOKeypointsDataProvider(DataProvider):
    URL = "https://cocodataset.org/#download"

    def __init__(self, root: str):
        self.task = "HumanPose"
        self.task_root = f"{root}/{self.task}"
        super().__init__(urls=_URLS, root=root)

    def move_to_raw_root(self):
        src_annots_path = f"{self.root}/annots"
        src_data_path = f"{self.root}/images"
        move(src_annots_path, f"{self.raw_root}/annots")
        move(src_data_path, f"{self.raw_root}/images")

    def _get_filepaths(self, dirnames: list[str] = ["annots", "images"]):
        super()._get_filepaths(dirnames)

    def _get_split_ids(self):
        test_idxs = None
        train_idxs = None
        val_idxs = None

        return {
            "train": train_idxs,
            "val": val_idxs,
            "test": test_idxs,
        }

    def arrange_files(self):
        Path(self.task_root).mkdir(parents=True, exist_ok=True)

        for split, idxs in self.split_ids.items():
            names = ["annots", "images"]
            dst_paths = {
                name: create_dir(Path(self.task_root) / name / split) for name in names
            }
            src_imgs_path = f"{self.raw_root}/images"
            dst_imgs_path = dst_paths["images"]
            dst_annots_path = dst_paths["annots"]

            filenames = [annot["filename"] for annot in split_annots]
            src_imgs_filepaths = [f"{src_imgs_path}/{fname}" for fname in filenames]
            dst_imgs_filepaths = [f"{dst_imgs_path}/{fname}" for fname in filenames]

            log.info(f"Moving {split} images from {src_imgs_path} to {dst_imgs_path}")
            copy_files(src_imgs_filepaths, dst_imgs_filepaths)

            log.info(f"Saving {split} annotations as .yaml files in {dst_annots_path}")
            for annot in split_annots:
                _id = annot["filename"].replace(".jpg", "")
                save_yaml(annot, f"{dst_annots_path}/{_id}.yaml")


if __name__ == "__main__":
    from geda.utils.config import ROOT

    root = str(ROOT / "data" / "COCO")
    dp = COCOKeypointsDataProvider(root)
    dp.get_data()



from geda.data_providers.base import SegmentationDataProvider
from geda.utils.files import (
    move,
    copy_files,
    save_txt_to_file,
    create_dir,
)
from geda.parsers.yolo import parse_segmentation_masks_to_yolo
from pathlib import Path
import glob
from geda.utils.pylogger import get_pylogger
import scipy
import mat73
from PIL import Image
from typing import Literal
import numpy as np
from tqdm.auto import tqdm

log = get_pylogger(__name__)


_URLS = {
    "nyu_depth_v2_labeled": "http://horatio.cs.nyu.edu/mit/silberman/nyu_depth_v2/nyu_depth_v2_labeled.mat",
    "splits": "http://horatio.cs.nyu.edu/mit/silberman/indoor_seg_sup/splits.mat",
}


class NYUDv2DataProvider(SegmentationDataProvider):
    URL = "https://cs.nyu.edu/~silberman/projects/indoor_scene_seg_sup.html"

    def __init__(self, root: str, labels_format: Literal["yolo"] | None = "yolo"):
        self.task = "Segmentation"
        self.task_root = f"{root}/{self.task}"
        super().__init__(urls=_URLS, root=root, labels_format=labels_format)

    def move_to_raw_root(self):
        src_data_path = f"{self.root}/nyu_depth_v2_labeled.mat"
        src_splits_path = f"{self.root}/splits.mat"
        copy_files([src_data_path], [self.raw_root])
        copy_files([src_splits_path], [self.raw_root])
        # move(src_data_path, self.raw_root) # TODO: move instead of copy? saves place
        # move(src_splits_path, self.raw_root)

    def _get_split_ids(self):
        splits = scipy.io.loadmat(f"{self.raw_root}/splits.mat")
        train_idxs = splits["trainNdxs"].flatten() - 1
        test_idxs = splits["testNdxs"].flatten() - 1
        return {
            "train": train_idxs.tolist(),
            "test": test_idxs.tolist(),
        }

    def arrange_files(self):
        log.info("Loading .mat file with data. It may take a while..")
        data_dict = mat73.loadmat(f"{self.raw_root}/nyu_depth_v2_labeled.mat")
        # H=480, W=640, N=1449
        numpy_images_data = {  # images to be saved as .npy files
            "semantic_masks": data_dict["labels"],  # HWN
            "instance_masks": data_dict["instances"],  # HWN
            "depth_masks": data_dict["depths"],  # HWN
        }

        images = data_dict["images"]  # HW3N

        classnames = ["unlabeled"] + [names[0] for names in data_dict["names"]]  # 1 + 894
        id2class = {i: _class for i, _class in enumerate(classnames)}
        self._set_id2class(id2class=id2class)
        scenes = [names[0] for names in data_dict["sceneTypes"]]  # N

        Path(self.task_root).mkdir(parents=True, exist_ok=True)

        for split, idxs in self.split_ids.items():
            names = list(numpy_images_data.keys()) + ["labels", "scenes", "images"]
            paths = {name: create_dir(Path(self.task_root) / name / split) for name in names}
            for name in names:
                log.info(f"Saving {split} {name} to {paths[name]}")

            for idx in tqdm(idxs, f"Saving {split} images"):
                for name, image_data in numpy_images_data.items():
                    img = image_data[..., idx]
                    np.save(f"{paths[name]}/{idx}.npy", img)

                Image.fromarray(images[..., idx]).save(f"{paths['images']}/{idx}.jpg")
                scene = scenes[idx]
                scene_filepath = f"{paths['scenes']}/{idx}.txt"
                save_txt_to_file(scene, scene_filepath)

    def create_labels(self):
        masks_filepaths = sorted(glob.glob(f"{self.task_root}/masks/*/*"))
        if self.labels_format == "yolo":
            parse_segmentation_masks_to_yolo(masks_filepaths, background=0, contour=None)
        else:
            log.warn(f"Only YOLO label format is implemented ({self.labels_format} passed)")

    def _get_class_counts(self):
        return super()._get_class_counts(masks_dirname="semantic_masks")

    def _get_filepaths(self):
        return super()._get_filepaths(
            dirnames=[
                "semantic_masks",
                "instance_masks",
                "scenes",
                "depths_masks",
                "images",
                "labels",
            ]
        )


if __name__ == "__main__":
    from geda.utils.config import ROOT

    root = str(ROOT / "data" / "NYUDv2")
    dp = NYUDv2DataProvider(root)



from geda.data_providers.base import DataProvider
from typing import Literal
from PIL import Image
from geda.utils.files import read_txt_file, load_yaml


class BaseDataset:
    def __init__(
        self,
        root: str,
        split: Literal["train", "val", "test"],
        data_provider: DataProvider,
        download: bool = True,
    ):
        if download:
            data_provider.get_data()
        self.data_provider = data_provider
        self.split = split
        self.ids = data_provider.split_ids[split]
        self.root = root


class BaseSegmentationDataset(BaseDataset):
    def __init__(
        self,
        root: str,
        split: Literal["train", "val", "test"],
        data_provider: DataProvider,
        download: bool = True,
    ):
        super().__init__(root, split, data_provider, download)
        self.images_paths = data_provider.filepaths["images"][split]
        self.masks_paths = data_provider.filepaths["masks"][split]

    def get_raw_data(self, idx) -> tuple[Image.Image, Image.Image]:
        image_fpath = self.images_paths[idx]
        mask_fpath = self.masks_paths[idx]
        image = Image.open(image_fpath).convert("RGB")
        mask = Image.open(mask_fpath)
        return image, mask

    def __len__(self):
        return len(self.images_paths)


class BaseClassificationDataset(BaseDataset):
    def __init__(
        self,
        root: str,
        split: Literal["train", "val", "test"],
        data_provider: DataProvider,
        download: bool = True,
    ):
        super().__init__(root, split, data_provider, download)
        print(data_provider.filepaths)
        self.images_paths = data_provider.filepaths["images"][split]
        self.labels_paths = data_provider.filepaths["labels"][split]

    def get_raw_data(self, idx: int) -> tuple[Image.Image, int]:
        image_fpath = self.images_paths[idx]
        label_fpath = self.labels_paths[idx]
        image = Image.open(image_fpath).convert("RGB")
        label = int(read_txt_file(label_fpath)[0])
        return image, label

    def __len__(self):
        return len(self.images_paths)


class BaseKeypointsDataset(BaseDataset):
    def __init__(
        self,
        root: str,
        split: Literal["train", "val", "test"],
        data_provider: DataProvider,
        download: bool = True,
    ):
        super().__init__(root, split, data_provider, download)
        print(data_provider.filepaths)
        self.images_paths = data_provider.filepaths["images"][split]
        self.annots_paths = data_provider.filepaths["annots"][split]

    def get_raw_data(self, idx: int) -> tuple[Image.Image, dict]:
        image_fpath = self.images_paths[idx]
        label_fpath = self.annots_paths[idx]
        image = Image.open(image_fpath).convert("RGB")
        annot = load_yaml(label_fpath)
        return image, annot

    def __len__(self):
        return len(self.images_paths)



from geda.data_providers.voc import (
    VOCInstanceSegmentationDataProvider,
    VOCPersonPartSegmentationDataProvider,
    VOCSemanticSegmentationDataProvider,
)
from geda.datasets.base import BaseSegmentationDataset
from typing import Literal


class VOCInstanceSegmentationDataset(BaseSegmentationDataset):
    def __init__(
        self, root: str, split: Literal["train", "val"], labels_format: Literal["yolo"] = "yolo"
    ):
        super().__init__(root, split, VOCInstanceSegmentationDataProvider(root, labels_format))


class VOCSemanticSegmentationDataset(BaseSegmentationDataset):
    def __init__(
        self, root: str, split: Literal["train", "val"], labels_format: Literal["yolo"] = "yolo"
    ):
        super().__init__(root, split, VOCSemanticSegmentationDataProvider(root, labels_format))


class VOCPersonPartSegmentationDataset(BaseSegmentationDataset):
    def __init__(
        self, root: str, split: Literal["train", "val"], labels_format: Literal["yolo"] = "yolo"
    ):
        super().__init__(root, split, VOCPersonPartSegmentationDataProvider(root, labels_format))


if __name__ == "__main__":
    from geda.utils.config import ROOT

    root = str(ROOT / "data" / "voc_2012")
    ds = VOCInstanceSegmentationDataset(root, "train")
